/**
 * JASMINe
 * Copyright (C) 2009-2010 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: NotificationModelLocator.as 6916 2010-09-20 15:38:18Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notification.model
{

import com.adobe.cairngorm.model.ModelLocator;

import flash.utils.Dictionary;

import mx.collections.ArrayCollection;
import mx.controls.*;
import mx.utils.UIDUtil;
import mx.collections.Sort;
import mx.collections.SortField;
import mx.controls.Alert;

import org.ow2.jasmine.monitoring.eos.notification.business.*;
import org.ow2.jasmine.monitoring.eos.notification.view.components.MultiFilterArrayCollection;
import org.ow2.jasmine.monitoring.eos.notification.vo.AcknowledgeStateVO;
import org.ow2.jasmine.monitoring.eos.notification.vo.ExecutedActionVO;
import org.ow2.jasmine.monitoring.eos.notification.vo.JasmineEventNotificationVO;
import org.ow2.jasmine.monitoring.eos.notification.vo.NotificationBoardSettingsVO;
import org.ow2.jasmine.monitoring.eos.notification.vo.NotificationSettingVO;


/**
 * @author radua
 */
 /**
  * The model locator for the notification module.
  */
[Bindable]
public class NotificationModelLocator implements ModelLocator
{
    public static function endsWith(source : String, suffix : String) : Boolean {
        var lastCharacters : int = source.length - suffix.length;
        if (lastCharacters >= 0) {
            if (source.lastIndexOf(suffix) == lastCharacters) {
                return true;
            }
        }
        return false;
    }

  /**
    * The unique ID of this component
    *
    * @internal
    *   Used to prevent a Cairngorm issue: when a command event is dispatched,
    * every controller that registered this event type receives it, even if
    * located in another module. To prevent this from happening and triggering
    * multiple severe unexpected concurrence bugs, each event dispatched is
    * postfixed with this unique ID.
    */
    public var componentID:String = UIDUtil.createUID();

    // public
    public static var NO_MESSAGE:uint = 0;

    public static var NEW_MESSAGE:uint = 1;


    // private

    /**
    * Unique instance of this locator.
    */
    private static var model:NotificationModelLocator = null;

    // jasmineNotificationObjs is for display, jasmineNotificationObjsIndex is for updates
    public var jasmineNotificationObjs:MultiFilterArrayCollection = new MultiFilterArrayCollection();
    public var jasmineNotificationObjsIndex:Dictionary = new Dictionary();

    private var stateObj:uint = NO_MESSAGE;

  /**
   * ArrayCollection of NotificationSettingVO. Contains:
   * - the name of the rule associated with the action.
   * - the name of the action.
   * - the level associated to the notification.
   * - whether the notification needs to be acknowledge before executing actions or not.
   *   Default is false.
   */
  private var actionsObj:ArrayCollection = null;

  /**
   * Binding between a rule and its level in the visualisation.
   *
   * Key    : String (the rule ID)
   * Object : LevelSettingVO
   */
  private var ruleLevels:Dictionary = new Dictionary();

  /**
   * Binding between a level and its associated color.
   *
   * Key    : String (the level name)
   * Object : String (the color, formatted this way : 0x000000)
   */
  public var levelColors:Dictionary = new Dictionary();

  /**
   * List of levels. Data provider of the level filter.
   */
  public var levels:ArrayCollection = new ArrayCollection();

  /**
   * List of allowed notifications.
   */
  private var allowedNotifications:ArrayCollection = new ArrayCollection();

  /**
  * contains all the notification settings.
  * Key   : rule Id of the notification
  * Value : NotificationSettingVO Objects.
  */
  public var notificationSettings : Dictionary = new Dictionary();

   /**
  * contains all the columns that will be displayed settings.
  * Key   : column Id in the configuration file
  * Value : ColumnSettingVO Objects.
  */
  //public var columnSettings : Dictionary = new Dictionary();

  private var columnsObj: ArrayCollection = null;

  public var showAllColumns:Boolean = false;

  /**
   * Whether all the notification are shown or only the set one.
   */
  public var showAllNotification:Boolean = false;

  /**
   * This contains the result of the last notification state checked.
   */
  private var lastStateCheckedObj : AcknowledgeStateVO;

  /**
   * The displayed name of the acknowledge button.
   */
  public var acknowledgeButtonName : String;

  /**
   * The last executed action.
   */
  private var updatedNotificationObj : ExecutedActionVO;

  /**
  * The logged user.
  */
  public var loggedUser : String;

    /**
     * How many hours the notification will be displayed after being acknowledged,
     * or if not synchronized after the notification date.
     */
    public var keepDisplayedHours : int;

    //
    //    Delegates
    //

    private var getJasmineNotifDelegate:IGetJasmineNotifDelegate = null;

    private var getNotificationSettingsDelegate:IGetNotificationSettingsDelegate = null;

    private var executeActionDelegate:IExecuteActionDelegate = null;

    private var getJasmineNotifJMSDelegate: IGetJasmineNotifJMSDelegate = null;

    private var checkNotificationStateDelegate:ICheckNotificationStateDelegate = null;

    private var getLoggedUserDelegate:IGetLoggedUserDelegate = null;

    private var getJasmineNotifFromToDelegate : IGetJasmineNotifFromToDelegate = null;

    // functions
    public function NotificationModelLocator()
    {
        super();

        if (model != null) {
            throw new Error("Only one NotificationModelLocator has to be set");
        }
    }

    public static function getInstance() : NotificationModelLocator {
        if (NotificationModelLocator.model == null) {
            NotificationModelLocator.model = new NotificationModelLocator();
        }

        return NotificationModelLocator.model;
    }

    // setters


  /**
   * Called when notifications are retrieved from the DataBase.
   */
    public function set jasmineNotifs(notifList:ArrayCollection) : void {

        this.jasmineNotificationObjs.filterFunctions = [];
        this.jasmineNotificationObjs.refresh();

        // when this function is called, this means that either we just started
        // the component, either we synchronized with DB, or either we reviewed
        // datas from DB. this means that we need to clear the list before
        // filling it with the new datas.
        this.jasmineNotificationObjs.removeAll();
        this.jasmineNotificationObjsIndex = new Dictionary();

        jasmineNotifsFromJMS = notifList;

        // by default, the board is sorted by dates.
        var sort : Sort = new Sort();
        var sortFieldDate : SortField = new SortField("date");
        sort.fields = [sortFieldDate];
        sort.reverse();
        this.jasmineNotificationObjs.sort = sort;
    }


    public function set state(state:uint) : void {
        this.stateObj = state;
    }


    /**
    * Called when notifications are retrieved from the JMS topic.
    */
    public function set jasmineNotifsFromJMS(notifList:ArrayCollection) : void {
        // if showAllNotification, then all notifications are shown
        if (this.showAllNotification) {
            for each (var notif:JasmineEventNotificationVO in notifList) {
                notif.level = this.ruleLevels[notif.rule];
                addUpdateNotification(notif);
            }
        } else {
            // Show only the set notifications
            for each (var _notif:JasmineEventNotificationVO in notifList) {
                for each (var rule:String in allowedNotifications) {
                    var match : Boolean = false;

                    if (endsWith(rule, "*")) {
                        if (_notif.rule.indexOf(rule.substring(0, rule.length - 1)) == 0) {
                            match = true;
                        }
                    } else {
                        match = (_notif.rule == rule);
                    }

                    if (match) {
                        _notif.level = this.ruleLevels[rule];
                        addUpdateNotification(_notif);
                        break;
                    }
                }
            }
        }
    }

    private function addUpdateNotification(notification : JasmineEventNotificationVO) : void {
        var oldNotification : JasmineEventNotificationVO = jasmineNotificationObjsIndex[notification.id]

        if (oldNotification == null) {
            jasmineNotificationObjsIndex[notification.id] = notification
            this.jasmineNotificationObjs.addItem(notification);
        } else {
            oldNotification.level = notification.level;
            oldNotification.date = notification.date;
            oldNotification.message = notification.message;
            oldNotification.status = notification.status;
            oldNotification.rule = notification.rule;
            oldNotification.details = notification.details;
            oldNotification.actions = notification.actions;
        }
    }

    // is still actionsObj useful ????
    //To be re-evaluated
    public function set actions(_action:ArrayCollection) : void {
      this.actionsObj = _action;
      // use to count the number of Notifications.
      var count:int = 0;
      for each (var a:NotificationSettingVO in this.actionsObj) {
        if (this.levelColors[a.level.level] == null) {
            this.levelColors[a.level.level] = a.level.color;
            this.levels.addItem({label:a.level.level});
        }

        this.ruleLevels[a.rule] = a.level;
        this.allowedNotifications.addItem(a.rule);
        this.notificationSettings[a.rule] = a;
        count++;
      }
      if (count == 0) {
          // no Notifications are set, that means that all the notification will be shown
          this.showAllNotification = true;
          this.keepDisplayedHours = Infinity;
      } else {
          this.showAllNotification = false;
          // set the acknowledge button name, as there is at least one notification in charge.
          this.acknowledgeButtonName = (this.actionsObj.getItemAt(0) as NotificationSettingVO).acknowledgeActionName;
          // set the number of hours an acknowledged notification should stay displayed in the board
          this.keepDisplayedHours = (this.actionsObj.getItemAt(0) as NotificationSettingVO).keepDisplayed;
      }
    }

  public function set columns(_columns: ArrayCollection) :void {
    this.columnsObj = _columns;
  }


  //A BIT DEPRECATED
  public function set notificationBoardSettings(_settings:NotificationBoardSettingsVO) : void {
      this.actionsObj = _settings.actionsList;
      this.columnsObj = _settings.columnsList;
      // use to count the number of Notifications.
      var count:int = 0;

      for each (var a:NotificationSettingVO in this.actionsObj) {
        this.ruleLevels[a.rule] = a.level;
        this.levelColors[a.level.level] = a.level.color;
        this.levels.addItem({label:a.level.level});
        this.allowedNotifications.addItem(a.rule);
        this.notificationSettings[a.rule] = a;
        count++;
      }
      if (count == 0) {
          // no Notifications are set, that means that all the notification will be shown
          this.showAllNotification = true;
          this.keepDisplayedHours = Infinity;
      } else {
          this.showAllNotification = false;
          // set the acknowledge button name, as there is at least one notification in charge.
          this.acknowledgeButtonName = (this.actionsObj.getItemAt(0) as NotificationSettingVO).acknowledgeActionName;
          // set the number of hours an acknowledged notification should stay displayed in the board
          this.keepDisplayedHours = (this.actionsObj.getItemAt(0) as NotificationSettingVO).keepDisplayed;
      }


      if (this.columnsObj.length == 0){
        this.showAllColumns = true;
      }


    }


    public function set availableNotifications(_notif:ArrayCollection) : void {
      // do nothing, this must not be used to update the list.
    }

    public function set lastStateChecked(_lastStateChecked:AcknowledgeStateVO) : void {
        this.lastStateCheckedObj = _lastStateChecked;
    }

    public function set updatedNotification(_updatedNotification : ExecutedActionVO) : void {
        this.updatedNotificationObj = _updatedNotification;
    }

    // getters

    public function get jasmineNotifs():ArrayCollection {
        return this.jasmineNotificationObjs;
    }

    public function get jasmineNotifsFromJMS() : ArrayCollection {
        return this.jasmineNotificationObjs;
    }

    public function get state() : uint {
        return this.stateObj;
    }

    public function get actions() : ArrayCollection {
      return this.actionsObj;
    }

    public function get availableNotifications() : ArrayCollection {
      return this.jasmineNotificationObjs;
    }

    public function get lastStateChecked() : AcknowledgeStateVO {
        return this.lastStateCheckedObj;
    }

    public function get updatedNotification() : ExecutedActionVO {
        return this.updatedNotificationObj;
    }

    public function get columns() : ArrayCollection {
      return this.columnsObj;
    }

    // getters delegates

   /**
    * getter for the delegate for the notifs in DB
    */
    public function getGetJasmineNotifDelegate():IGetJasmineNotifDelegate {
        if (this.getJasmineNotifDelegate == null) {
            this.getJasmineNotifDelegate = new GetJasmineNotifDelegate();
        }
        return this.getJasmineNotifDelegate;
    }


    public function getGetNotificationSettingsDelegate():IGetNotificationSettingsDelegate {
        if (this.getNotificationSettingsDelegate == null) {
            this.getNotificationSettingsDelegate = new GetNotificationSettingsDelegate();
        }
        return this.getNotificationSettingsDelegate;
    }

    public function getExecuteActionDelegate():IExecuteActionDelegate {
        if (this.executeActionDelegate == null) {
            this.executeActionDelegate = new ExecuteActionDelegate();
        }
        return this.executeActionDelegate;
    }

   /**
    * getter for the delegate for the notifs in JMS topic
    */
     public function getGetJasmineNotifJMSDelegate():IGetJasmineNotifJMSDelegate {
        if (this.getJasmineNotifJMSDelegate == null) {
            this.getJasmineNotifJMSDelegate = new GetJasmineNotifJMSDelegate();
        }
        return this.getJasmineNotifJMSDelegate;
    }

    /**
    * Getter for the delegate that checks the notification state.
    */
    public function getCheckNotificationStateDelegate():ICheckNotificationStateDelegate {
        if (this.checkNotificationStateDelegate == null) {
            this.checkNotificationStateDelegate = new CheckNotificationStateDelegate();
        }
        return this.checkNotificationStateDelegate;
    }

    public function getGetLoggedUserDelegate():IGetLoggedUserDelegate {
        if (this.getLoggedUserDelegate == null) {
            this.getLoggedUserDelegate = new GetLoggedUserDelegate();
        }
        return this.getLoggedUserDelegate;
    }

    public function getGetJasmineNotifFromToDelegate() : IGetJasmineNotifFromToDelegate {
        if (this.getJasmineNotifFromToDelegate == null) {
            this.getJasmineNotifFromToDelegate = new GetJasmineNotifFromToDelegate();
        }
        return this.getJasmineNotifFromToDelegate;
    }

}
}
